#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 13 11:46:01 2023

@author: jaspermittag
"""

import random
import numpy as np
import pygame

pygame.init()

WIDTH, HEIGHT = 700, 400
WIN = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("Brownsche Bewegung")

RED = (255, 82, 32)
BLACK = (0, 0, 0)
BLUE = (23, 2, 255)
WHITE = (255, 255, 255)
GREEN = (50, 205, 50)
YELLOW = (255, 255, 0)

FPS = 60

ballAmount = 100
ballRadius = 5
ballColor = BLUE
ballMasse = 10
SPEED = 2

BROWNSCHES_TEILCHEN_COLOR = RED
BROWNSCHES_TEILCHEN_RADIUS = 20
BROSCNHES_TEILCHEN_MASSE = 50



class Teilchen:
    def __init__(self, color, radius, masse):
        self.color = color
        self.radius = radius
        self.x = random.randint(0 + radius, WIDTH - radius)
        self.y = random.randint(0 + radius, HEIGHT - radius)
        self.xy = np.array([self.x, self.y])
        self.vxy = np.array([1, 1]) * SPEED
        self.xydirection = np.array([random.randint(-1, 1), random.randint(-1, 1)])
        self.masse = masse
        
    def draw(self, win):
        '''zeichnet das Teilchen ins Fenster'''
        pygame.draw.circle(win, self.color, (self.x, self.y), self.radius)
        
    def move(self):
        self.x += self.vxy[0] * self.xydirection[0]
        self.y += self.vxy[1] * self.xydirection[1]
      
    # Kollision mit einer Wand     
    def borderCollision(self):
        if self.x >= WIDTH - self.radius:
          self.x = WIDTH - self.radius
          self.xydirection[0] *= -1
        elif self.x <= 0 + self.radius:
          self.x = self.radius
          self.xydirection[0] *= -1
        if self.y >= HEIGHT - self.radius:
          self.y = HEIGHT - self.radius
          self.xydirection[1] *= -1
        elif self.y <= 0 + self.radius:
          self.y = self.radius
          self.xydirection[1] *= -1
    
    def ballCollision(self, ball2):
        sv = self.xy - ball2.xy
        distance = np.sqrt(sv[0]**2 + sv[1]**2)
        
        if distance <= self.radius * 2:
            vxy_ball = self.vxy
            self.vxy = (self.masse * self.vxy + ball2.masse *
                         (2 * self.vxy - ball2.vxy)) / (self.masse + ball2.masse)
            ball2.vxy = (ball2.masse * ball2.vxy + self.masse *
                         (2 * ball2.vxy - vxy_ball)) / (self.masse + ball2.masse)
             
            b1_dir = self.xydirection
            self.xydirection = ball2.xydirection 
            ball2.xydirection = b1_dir
        
            
          
         
        
def draw(win, balls):
    win.fill(BLACK)
    for ball in balls:
        ball.draw(win)
    pygame.display.update()
    
                  

def main(): 
    run = 1
    
    # Teilchenobjekte in eine Liste schreiben
    balls = []
    for i in range(ballAmount):
        ball = "ball"+str(i)
        ball = Teilchen(ballColor, ballRadius, ballMasse)
        balls.append(ball)
    
    brownschesTeilchen = Teilchen(BROWNSCHES_TEILCHEN_COLOR, BROWNSCHES_TEILCHEN_RADIUS, 
                                  BROSCNHES_TEILCHEN_MASSE)
    balls.append(brownschesTeilchen)
    
    while run:
        draw(WIN, balls)
        pygame.time.Clock().tick(FPS)
        
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                run = 0
                break
            
        for ball in balls:
            ball.borderCollision()
            for i in range(balls.index(ball), len(balls)): 
                ball.ballCollision(balls[i])                
            ball.move()

        
    pygame.quit()


if __name__ == '__main__':
    main()